@@ -1,7 +1,7 @@
-export CONDA_EXE="$(cygpath 'C:\bld\conda_1554117451804\_h_env\Scripts\conda.exe')"
+export CONDA_EXE="$(cygpath 'C:\bld\conda_1554240139111\_h_env\Scripts\conda.exe')"
 export _CE_M=-m
 export _CE_CONDA=conda
-export CONDA_BAT="C:\bld\conda_1554117451804\_h_env\condabin\conda.bat"
+export CONDA_BAT="C:\bld\conda_1554240139111\_h_env\condabin\conda.bat"
 # Copyright (C) 2012 Anaconda, Inc
 # SPDX-License-Identifier: BSD-3-Clause
 
@@ -58,15 +58,7 @@ conda() {
 
 if [ -z "${CONDA_SHLVL+x}" ]; then
     \export CONDA_SHLVL=0
-    if [ "${_CE_CONDA+x}" == "condax" ]; then
-        if [ "${PATH+x}" == "x" ]; then
-            PATH="$(dirname "$CONDA_EXE")/condabin"
-        else
-            PATH="$(dirname "$(dirname "$CONDA_EXE")")/condabin:${PATH}"
-        fi
-    else
-        PATH="$(dirname "$(dirname "$CONDA_EXE")")/condabin:${PATH:-}"
-    fi
+    PATH="$(\dirname "$(\dirname "$CONDA_EXE")")/condabin${PATH:+":${PATH}"}"
     \export PATH
 
     # We're not allowing PS1 to be unbound. It must at least be set.
