@@ -3,19 +3,21 @@
 # SPDX-License-Identifier: BSD-3-Clause
 from __future__ import absolute_import, division, print_function, unicode_literals
 
+from collections import OrderedDict
 from errno import ENOENT
 from glob import glob
+from itertools import chain
 import os
 from os.path import abspath, basename, dirname, expanduser, expandvars, isdir, join
 import re
 import sys
-from tempfile import NamedTemporaryFile
 from textwrap import dedent
 
 # Since we have to have configuration context here, anything imported by
 #   conda.base.context is fair game, but nothing more.
 from . import CONDA_PACKAGE_ROOT, CondaError
 from ._vendor.toolz import concatv, drop
+from ._vendor.auxlib.compat import Utf8NamedTemporaryFile
 from .base.context import ROOT_ENV_NAME, context, locate_prefix_by_name
 from .common.compat import FILESYSTEM_ENCODING, PY2, iteritems, on_win, string_types, text_type
 from .common.path import paths_equal
@@ -67,15 +69,76 @@ class _Activator(object):
         else:
             self.environ = os.environ.copy()
 
+    # Once Python2 dies odargs can become kwargs again since dicts are ordered since 3.6.
+    def get_export_unset_vars(self, odargs):
+        """
+        :param kwargs: environment variables to export. The `conda_exe_vars` meta
+                       variables are also exported by default. If you do not want
+                       this to happen then pass:
+                           conda_exe_vars=None
+                       .. if you pass and set any other variable to None, then it
+                       emits it to the dict with a value of None.
+
+        :return: A OrderedDict of env vars to export ordered the same way as kwargs.
+                 And a list of env vars to unset.
+        """
+        kwargs = odargs
+        conda_exe_vars_None = False if ('conda_exe_vars' not in kwargs or
+                                        kwargs['conda_exe_vars'] is not None) else True
+        conda_exe_unset_vars = []
+        unset_vars = []
+        # conda_exe_vars = context.conda_exe_vars_dict.copy()
+        from collections import OrderedDict
+        conda_exe_vars_export = OrderedDict()
+        for k, v in context.conda_exe_vars_dict.items():
+            if v is None or conda_exe_vars_None:
+                conda_exe_unset_vars.append(k)
+            else:
+                conda_exe_vars_export[k] = self.path_conversion(v) if v else v
+        from collections import OrderedDict
+        export_vars = OrderedDict()
+        for k, v in kwargs.items():
+            if k == 'conda_exe_vars':
+                continue
+            elif v is None:
+                unset_vars.append(k.upper())
+            else:
+                export_vars[k.upper()] = v
+        # Just to make writing tests a bit more ergonomic we add these to the end.
+        unset_vars += conda_exe_unset_vars
+        export_vars = OrderedDict(chain(export_vars.items(), conda_exe_vars_export.items()))
+        return export_vars, unset_vars
+
+    # Used in tests only.
+    def add_export_unset_vars(self, export_vars, unset_vars, **kwargs):
+        new_export_vars, new_unset_vars = self.get_export_unset_vars(odargs=OrderedDict(kwargs))
+        if export_vars is not None:
+            export_vars = OrderedDict(chain(export_vars.items(), new_export_vars.items()))
+        if unset_vars is not None:
+            unset_vars += new_unset_vars
+        return export_vars, unset_vars
+
+    # Used in tests only.
+    def get_scripts_export_unset_vars(self, **kwargs):
+        export_vars, unset_vars = self.get_export_unset_vars(odargs=OrderedDict(kwargs))
+        script_export_vars = script_unset_vars = None
+        if export_vars:
+            script_export_vars = self.command_join.join(
+                [self.export_var_tmpl % (k, v) for k, v in export_vars.items()])
+        if unset_vars:
+            script_unset_vars = self.command_join.join(
+                [self.unset_var_tmpl % (k) for k in unset_vars])
+        return script_export_vars or '', script_unset_vars or ''
+
     def _finalize(self, commands, ext):
         commands = concatv(commands, ('',))  # add terminating newline
         if ext is None:
             return self.command_join.join(commands)
         elif ext:
-            with NamedTemporaryFile('w+b', suffix=ext, delete=False) as tf:
+            with Utf8NamedTemporaryFile('w+', suffix=ext, delete=False) as tf:
                 # the default mode is 'w+b', and universal new lines don't work in that mode
                 # command_join should account for that
-                tf.write(ensure_binary(self.command_join.join(commands)))
+                tf.write(self.command_join.join(commands))
             return tf.name
         else:
             raise NotImplementedError()
@@ -161,6 +224,15 @@ class _Activator(object):
             from .exceptions import ArgumentError
             raise ArgumentError("invalid command '%s'" % command)
 
+        if command.endswith('activate') or command == 'hook':
+            try:
+                dev_idx = remainder_args.index('--dev')
+            except ValueError:
+                context.dev = False
+            else:
+                del remainder_args[dev_idx]
+                context.dev = True
+
         if command == 'activate':
             try:
                 stack_idx = remainder_args.index('--stack')
@@ -187,13 +259,13 @@ class _Activator(object):
         for script in cmds_dict.get('deactivate_scripts', ()):
             yield self.run_script_tmpl % script
 
-        for key in sorted(cmds_dict.get('unset_vars', ())):
+        for key in cmds_dict.get('unset_vars', ()):
             yield self.unset_var_tmpl % key
 
-        for key, value in sorted(iteritems(cmds_dict.get('set_vars', {}))):
+        for key, value in iteritems(cmds_dict.get('set_vars', {})):
             yield self.set_var_tmpl % (key, value)
 
-        for key, value in sorted(iteritems(cmds_dict.get('export_vars', {}))):
+        for key, value in iteritems(cmds_dict.get('export_vars', {})):
             yield self.export_var_tmpl % (key, value)
 
         for script in cmds_dict.get('activate_scripts', ()):
@@ -228,19 +300,16 @@ class _Activator(object):
         conda_default_env = self._default_env(prefix)
         conda_prompt_modifier = self._prompt_modifier(prefix, conda_default_env)
 
+        unset_vars = []
         if old_conda_shlvl == 0:
             new_path = self.pathsep_join(self._add_prefix_to_path(prefix))
-            conda_python_exe = self.path_conversion(sys.executable)
-            conda_exe = self.path_conversion(context.conda_exe)
-            export_vars = {
-                'CONDA_PYTHON_EXE': conda_python_exe,
-                'CONDA_EXE': conda_exe,
-                'PATH': new_path,
-                'CONDA_PREFIX': prefix,
-                'CONDA_SHLVL': new_conda_shlvl,
-                'CONDA_DEFAULT_ENV': conda_default_env,
-                'CONDA_PROMPT_MODIFIER': conda_prompt_modifier,
-            }
+            export_vars, unset_vars = self.get_export_unset_vars(
+                odargs=OrderedDict((
+                    ('path', new_path),
+                    ('conda_prefix', prefix),
+                    ('conda_shlvl', new_conda_shlvl),
+                    ('conda_default_env', conda_default_env),
+                    ('conda_prompt_modifier', conda_prompt_modifier))))
             deactivate_scripts = ()
         else:
             if self.environ.get('CONDA_PREFIX_%s' % (old_conda_shlvl - 1)) == prefix:
@@ -249,29 +318,26 @@ class _Activator(object):
                 return self.build_deactivate()
             if stack:
                 new_path = self.pathsep_join(self._add_prefix_to_path(prefix))
-                export_vars = {
-                    'PATH': new_path,
-                    'CONDA_PREFIX': prefix,
-                    'CONDA_PREFIX_%d' % old_conda_shlvl: old_conda_prefix,
-                    'CONDA_SHLVL': new_conda_shlvl,
-                    'CONDA_DEFAULT_ENV': conda_default_env,
-                    'CONDA_PROMPT_MODIFIER': conda_prompt_modifier,
-                    'CONDA_STACKED_%d' % new_conda_shlvl: 'true',
-                }
                 deactivate_scripts = ()
+                export_vars, unset_vars = self.get_export_unset_vars(odargs=OrderedDict((
+                    ('path', new_path),
+                    ('conda_prefix', prefix),
+                    ('conda_shlvl', new_conda_shlvl),
+                    ('conda_default_env', conda_default_env),
+                    ('conda_prompt_modifier', conda_prompt_modifier))))
+                export_vars['CONDA_PREFIX_%d' % old_conda_shlvl] = old_conda_prefix
+                export_vars['CONDA_STACKED_%d' % new_conda_shlvl] = 'true'
             else:
                 new_path = self.pathsep_join(
-                    self._replace_prefix_in_path(old_conda_prefix, prefix)
-                )
-                export_vars = {
-                    'PATH': new_path,
-                    'CONDA_PREFIX': prefix,
-                    'CONDA_PREFIX_%d' % old_conda_shlvl: old_conda_prefix,
-                    'CONDA_SHLVL': new_conda_shlvl,
-                    'CONDA_DEFAULT_ENV': conda_default_env,
-                    'CONDA_PROMPT_MODIFIER': conda_prompt_modifier,
-                }
+                    self._replace_prefix_in_path(old_conda_prefix, prefix))
                 deactivate_scripts = self._get_deactivate_scripts(old_conda_prefix)
+                export_vars, unset_vars = self.get_export_unset_vars(odargs=OrderedDict((
+                    ('path', new_path),
+                    ('conda_prefix', prefix),
+                    ('conda_shlvl', new_conda_shlvl),
+                    ('conda_default_env', conda_default_env),
+                    ('conda_prompt_modifier', conda_prompt_modifier))))
+                export_vars['CONDA_PREFIX_%d' % old_conda_shlvl] = old_conda_prefix
 
         set_vars = {}
         if context.changeps1:
@@ -280,7 +346,7 @@ class _Activator(object):
         self._build_activate_shell_custom(export_vars)
 
         return {
-            'unset_vars': (),
+            'unset_vars': unset_vars,
             'set_vars': set_vars,
             'export_vars': export_vars,
             'deactivate_scripts': deactivate_scripts,
@@ -295,8 +361,8 @@ class _Activator(object):
             # no active environment, so cannot deactivate; do nothing
             return {
                 'unset_vars': (),
-                'set_vars': {},
-                'export_vars': {},
+                'set_vars': OrderedDict(),
+                'export_vars': OrderedDict(),
                 'deactivate_scripts': (),
                 'activate_scripts': (),
             }
@@ -306,17 +372,20 @@ class _Activator(object):
         set_vars = {}
         if old_conda_shlvl == 1:
             new_path = self.pathsep_join(self._remove_prefix_from_path(old_conda_prefix))
+            # You might think that you can remove the CONDA_EXE vars by passing conda_exe_vars=None
+            # here so that "deactivate means deactivate" but you cannot since the conda shell
+            # scripts still refer to them and they only set them once at the top. We could change
+            # that though, the conda() shell function could set them instead of doing it at the
+            # top.  This would be *much* cleaner. I personally cannot abide that I have
+            # deactivated conda and anything at all in my env still references it (apart from the
+            # shell script, we need something I suppose!)
+            export_vars, unset_vars = self.get_export_unset_vars(odargs=OrderedDict((
+                ('path', new_path),
+                ('conda_prefix', None),
+                ('conda_shlvl', new_conda_shlvl),
+                ('conda_default_env', None),
+                ('conda_prompt_modifier', None))))
             conda_prompt_modifier = ''
-            unset_vars = (
-                'CONDA_PREFIX',
-                'CONDA_DEFAULT_ENV',
-                'CONDA_PYTHON_EXE',
-                'CONDA_PROMPT_MODIFIER',
-            )
-            export_vars = {
-                'PATH': new_path,
-                'CONDA_SHLVL': new_conda_shlvl,
-            }
             activate_scripts = ()
         else:
             assert old_conda_shlvl > 1
@@ -325,27 +394,25 @@ class _Activator(object):
             conda_prompt_modifier = self._prompt_modifier(new_prefix, conda_default_env)
 
             old_prefix_stacked = 'CONDA_STACKED_%d' % old_conda_shlvl in self.environ
+            new_path = ''
+
+            unset_vars = ['CONDA_PREFIX_%d' % new_conda_shlvl]
             if old_prefix_stacked:
                 new_path = self.pathsep_join(self._remove_prefix_from_path(old_conda_prefix))
-                unset_vars = (
-                    'CONDA_PREFIX_%d' % new_conda_shlvl,
-                    'CONDA_STACKED_%d' % old_conda_shlvl,
-                )
+                unset_vars.append('CONDA_STACKED_%d' % old_conda_shlvl)
             else:
                 new_path = self.pathsep_join(
                     self._replace_prefix_in_path(old_conda_prefix, new_prefix)
                 )
-                unset_vars = (
-                    'CONDA_PREFIX_%d' % new_conda_shlvl,
-                )
 
-            export_vars = {
-                'PATH': new_path,
-                'CONDA_SHLVL': new_conda_shlvl,
-                'CONDA_PREFIX': new_prefix,
-                'CONDA_DEFAULT_ENV': conda_default_env,
-                'CONDA_PROMPT_MODIFIER': conda_prompt_modifier,
-            }
+            export_vars, unset_vars2 = self.get_export_unset_vars(odargs=OrderedDict((
+                ('path', new_path),
+                ('conda_prefix', new_prefix),
+                ('conda_shlvl', new_conda_shlvl),
+                ('conda_default_env', conda_default_env),
+                ('conda_prompt_modifier', conda_prompt_modifier))))
+            unset_vars += unset_vars2
+
             activate_scripts = self._get_activate_scripts(new_prefix)
 
         if context.changeps1:
@@ -366,8 +433,8 @@ class _Activator(object):
             # no active environment, so cannot reactivate; do nothing
             return {
                 'unset_vars': (),
-                'set_vars': {},
-                'export_vars': {},
+                'set_vars': OrderedDict(),
+                'export_vars': OrderedDict(),
                 'deactivate_scripts': (),
                 'activate_scripts': (),
             }
@@ -383,50 +450,52 @@ class _Activator(object):
         return {
             'unset_vars': (),
             'set_vars': set_vars,
-            'export_vars': {
-                'PATH': new_path,
-                'CONDA_SHLVL': conda_shlvl,
-                'CONDA_PROMPT_MODIFIER': self._prompt_modifier(conda_prefix, conda_default_env),
-            },
+            'export_vars': OrderedDict([('PATH', new_path),
+                                        ('CONDA_SHLVL', conda_shlvl),
+                                        ('CONDA_PROMPT_MODIFIER', self._prompt_modifier(
+                                            conda_prefix, conda_default_env))]),
             'deactivate_scripts': self._get_deactivate_scripts(conda_prefix),
             'activate_scripts': self._get_activate_scripts(conda_prefix),
         }
 
     def _get_starting_path_list(self):
-        path = self.environ.get('PATH', '')
+        # For isolation, running the conda test suite *without* env. var. inheritance
+        # every so often is a good idea. We should probably make this a pytest fixture
+        # along with one that tests both hardlink-only and copy-only, but before that
+        # conda's testsuite needs to be a lot faster!
+        clean_paths = {'darwin': '/usr/bin:/bin:/usr/sbin:/sbin',
+                       # You may think 'let us do something more clever here and interpolate
+                       # `%windir%`' but the point here is the the whole env. is cleaned out
+                       'win32': 'C:\\Windows\\system32;'
+                                'C:\\Windows;'
+                                'C:\\Windows\\System32\\Wbem;'
+                                'C:\\Windows\\System32\\WindowsPowerShell\\v1.0\\'
+                       }
+        path = self.environ.get('PATH',
+                                clean_paths[sys.platform] if sys.platform in clean_paths else
+                                '/usr/bin')
+        path_split = path.split(os.pathsep)
         if on_win:
-            # On Windows, the Anaconda Python interpreter prepends sys.prefix\Library\bin on
-            # startup. It's a hack that allows users to avoid using the correct activation
-            # procedure; a hack that needs to go away because it doesn't add all the paths.
-            # See: https://github.com/AnacondaRecipes/python-feedstock/blob/master/recipe/0005-Win32-Ensure-Library-bin-is-in-os.environ-PATH.patch  # NOQA
-            # But, we now detect if that has happened because:
-            #   1. In future we would like to remove this hack and require real activation.
-            #   2. We should not assume that the Anaconda Python interpreter is being used.
-            path_split = path.split(os.pathsep)
-            library_bin = r"%s\Library\bin" % (sys.prefix)
-            # ^^^ deliberately the same as: https://github.com/AnacondaRecipes/python-feedstock/blob/8e8aee4e2f4141ecfab082776a00b374c62bb6d6/recipe/0005-Win32-Ensure-Library-bin-is-in-os.environ-PATH.patch#L20  # NOQA
-            if paths_equal(path_split[0], library_bin):
-                return path_split[1:]
-            else:
-                return path_split
-        else:
-            return path.split(os.pathsep)
+            # We used to prepend sys.prefix\Library\bin to PATH on startup but not anymore.
+            # Instead, in conda 4.6 we add the full suite of entries. This is performed in
+            # condabin\conda.bat and condabin\ _conda_activate.bat. However, we
+            # need to ignore the stuff we add there, and only consider actual PATH entries.
+            prefix_dirs = tuple(self._get_path_dirs(sys.prefix))
+            start_index = 0
+            while (start_index < len(prefix_dirs) and
+                   start_index < len(path_split) and
+                   paths_equal(path_split[start_index], prefix_dirs[start_index])):
+                start_index += 1
+            path_split = path_split[start_index:]
+            library_bin_dir = self.path_conversion(
+                    self.sep.join((sys.prefix, 'Library', 'bin')))
+            if paths_equal(path_split[0], library_bin_dir):
+                path_split = path_split[1:]
+        return path_split
 
-    @staticmethod
-    def _get_path_dirs(prefix):
+    def _get_path_dirs(self, prefix, extra_library_bin=False):
         if on_win:  # pragma: unix no cover
             yield prefix.rstrip("\\")
-            yield join(prefix, 'Library', 'mingw-w64', 'bin')
-            yield join(prefix, 'Library', 'usr', 'bin')
-            yield join(prefix, 'Library', 'bin')
-            yield join(prefix, 'Scripts')
-            yield join(prefix, 'bin')
-        else:
-            yield join(prefix, 'bin')
-
-    def _get_path_dirs2(self, prefix):
-        if on_win:  # pragma: unix no cover
-            yield prefix
             yield self.sep.join((prefix, 'Library', 'mingw-w64', 'bin'))
             yield self.sep.join((prefix, 'Library', 'usr', 'bin'))
             yield self.sep.join((prefix, 'Library', 'bin'))
@@ -451,7 +520,7 @@ class _Activator(object):
             condabin_dir = self.path_conversion(join(context.conda_prefix, "condabin"))
             path_list.insert(0, condabin_dir)
 
-        path_list[0:0] = list(self.path_conversion(self._get_path_dirs2(prefix)))
+        path_list[0:0] = list(self.path_conversion(self._get_path_dirs(prefix)))
         return tuple(path_list)
 
     def _remove_prefix_from_path(self, prefix, starting_path_dirs=None):
@@ -472,19 +541,25 @@ class _Activator(object):
             return None
 
         if old_prefix is not None:
-            prefix_dirs = tuple(self._get_path_dirs2(old_prefix))
+            prefix_dirs = tuple(self._get_path_dirs(old_prefix))
             first_idx = index_of_path(path_list, prefix_dirs[0])
             if first_idx is None:
                 first_idx = 0
             else:
                 last_idx = index_of_path(path_list, prefix_dirs[-1])
                 assert last_idx is not None
+                # this compensates for an extra Library/bin dir entry from the interpreter on
+                #     windows.  If that entry isn't being added, it should have no effect.
+                library_bin_dir = self.path_conversion(
+                    self.sep.join((sys.prefix, 'Library', 'bin')))
+                if path_list[last_idx + 1] == library_bin_dir:
+                    last_idx += 1
                 del path_list[first_idx:last_idx + 1]
         else:
             first_idx = 0
 
         if new_prefix is not None:
-            path_list[first_idx:first_idx] = list(self._get_path_dirs2(new_prefix))
+            path_list[first_idx:first_idx] = list(self._get_path_dirs(new_prefix))
 
         return tuple(path_list)
 
@@ -527,13 +602,8 @@ def expand(path):
 
 
 def ensure_binary(value):
-    encoding = 'utf-8'
-    if on_win:
-        import ctypes
-        acp = ctypes.cdll.kernel32.GetACP()
-        encoding = str(acp)
     try:
-        return value.encode(encoding)
+        return value.encode('utf-8')
     except AttributeError:  # pragma: no cover
         # AttributeError: '<>' object has no attribute 'encode'
         # In this case assume already binary type and do nothing
@@ -554,8 +624,16 @@ def native_path_to_unix(paths):  # pragma: unix no cover
     if paths is None:
         return None
     from subprocess import CalledProcessError, PIPE, Popen
-    from shlex import split
-    command = 'cygpath --path -f -'
+    from conda._vendor.auxlib.compat import shlex_split_unicode
+    # It is very easy to end up with a bash in one place and a cygpath in another due to e.g.
+    # using upstream MSYS2 bash, but with a conda env that does not have bash but does have
+    # cygpath.  When this happens, we have two different virtual POSIX machines, rooted at
+    # different points in the Windows filesystem.  We do our path conversions with one and
+    # expect the results to work with the other.  It does not.
+    from .common.path import which
+    bash = which('bash')
+    command = os.path.join(dirname(bash), 'cygpath') if bash else 'cygpath'
+    command += ' --path -f -'
 
     single_path = isinstance(paths, string_types)
     joined = paths if single_path else ("%s" % os.pathsep).join(paths)
@@ -564,7 +642,7 @@ def native_path_to_unix(paths):  # pragma: unix no cover
         joined = joined.encode('utf-8')
 
     try:
-        p = Popen(split(command), stdin=PIPE, stdout=PIPE, stderr=PIPE)
+        p = Popen(shlex_split_unicode(command), stdin=PIPE, stdout=PIPE, stderr=PIPE)
     except EnvironmentError as e:
         if e.errno != ENOENT:
             raise
@@ -612,10 +690,10 @@ class PosixActivator(_Activator):
         self.tempfile_extension = None  # write instructions to stdout rather than a temp file
         self.command_join = '\n'
 
-        self.unset_var_tmpl = '\\unset %s'
-        self.export_var_tmpl = "\\export %s='%s'"
+        self.unset_var_tmpl = 'unset %s'
+        self.export_var_tmpl = "export %s='%s'"
         self.set_var_tmpl = "%s='%s'"
-        self.run_script_tmpl = '\\. "%s"'
+        self.run_script_tmpl = '. "%s"'
 
         self.hook_source_path = join(CONDA_PACKAGE_ROOT, 'shell', 'etc', 'profile.d', 'conda.sh')
 
@@ -639,12 +717,31 @@ class PosixActivator(_Activator):
 
     def _hook_preamble(self):
         if on_win:
-            return ('export CONDA_EXE="$(cygpath \'%s\')"\n'
-                    'export CONDA_BAT="%s"'
-                    % (context.conda_exe, join(context.conda_prefix, 'condabin', 'conda.bat'))
-                    )
+            if context.dev:
+                return ('export CONDA_EXE="$(cygpath \'%s\')"\n'
+                        'export _CE_M=-m\n'
+                        'export _CE_CONDA=conda\n'
+                        'export PYTHONPATH="$(cygpath \'%s\')"\n'
+                        % (sys.executable, dirname(CONDA_PACKAGE_ROOT))
+                        )
+            else:
+                return ('export CONDA_EXE="$(cygpath \'%s\')"\n'
+                        'export _CE_M=-m\n'
+                        'export _CE_CONDA=conda\n'
+                        'export CONDA_BAT="%s"'
+                        % (context.conda_exe, join(context.conda_prefix, 'condabin', 'conda.bat'))
+                        )
         else:
-            return 'export CONDA_EXE="%s"' % context.conda_exe
+            result = ''
+            for key, value in context.conda_exe_vars_dict.items():
+                if value is None:
+                    # Using `unset_var_tmpl` would cause issues for people running
+                    # with shell flag -u set (error on unset).
+                    # result += join(self.unset_var_tmpl % key) + '\n'
+                    result += join(self.export_var_tmpl % (key, '')) + '\n'
+                else:
+                    result += join(self.export_var_tmpl % (key, value)) + '\n'
+            return result
 
 
 class CshActivator(_Activator):
@@ -719,7 +816,7 @@ class CmdExeActivator(_Activator):
         self.path_conversion = path_identity
         self.script_extension = '.bat'
         self.tempfile_extension = '.bat'
-        self.command_join = '\r\n' if on_win else '\n'
+        self.command_join = '\n'
 
         self.unset_var_tmpl = '@SET %s='
         self.export_var_tmpl = '@SET "%s=%s"'
@@ -732,15 +829,9 @@ class CmdExeActivator(_Activator):
 
         super(CmdExeActivator, self).__init__(arguments)
 
-    def _build_activate_shell_custom(self, export_vars):
-        if on_win:
-            import ctypes
-            export_vars.update({
-                "PYTHONIOENCODING": ctypes.cdll.kernel32.GetACP(),
-            })
-
-    def _hook_preamble(self):
-        raise NotImplementedError()
+    # def _hook_preamble(self):
+    #     if on_win:
+    #         return '@chcp 65001'
 
 
 class FishActivator(_Activator):
@@ -780,7 +871,7 @@ class PowerShellActivator(_Activator):
 
     def __init__(self, arguments=None):
         self.pathsep_join = ';'.join if on_win else ':'.join
-        self.sep = '/'  # Even on Windows, PowerShell can handle Unix-style separators.
+        self.sep = '\\' if on_win else '/'
         self.path_conversion = path_identity
         self.script_extension = '.ps1'
         self.tempfile_extension = None  # write instructions to stdout rather than a temp file
@@ -796,11 +887,22 @@ class PowerShellActivator(_Activator):
         super(PowerShellActivator, self).__init__(arguments)
 
     def _hook_preamble(self):
-        return dedent("""\
-        $Env:CONDA_EXE = "{context.conda_exe}"
-        $Env:_CONDA_ROOT = "{context.conda_prefix}"
-        $Env:_CONDA_EXE = "{context.conda_exe}"
-        """.format(context=context))
+        if context.dev:
+            return dedent("""\
+                $Env:CONDA_EXE = "{context.conda_exe}"
+                $Env:_CE_M = "-m"
+                $Env:_CE_CONDA = "conda"
+                $Env:_CONDA_ROOT = "{context.conda_prefix}"
+                $Env:_CONDA_EXE = "{context.conda_exe}"
+                """.format(context=context))
+        else:
+            return dedent("""\
+                $Env:CONDA_EXE = "{context.conda_exe}"
+                $Env:_CE_M = ""
+                $Env:_CE_CONDA = ""
+                $Env:_CONDA_ROOT = "{context.conda_prefix}"
+                $Env:_CONDA_EXE = "{context.conda_exe}"
+                """.format(context=context))
 
 
 activator_map = {
