@@ -10,7 +10,7 @@ from os.path import abspath, basename, expanduser, expandvars, join, normcase, s
 import re
 import subprocess
 
-from .compat import PY2, ensure_fs_path_encoding, on_win, string_types
+from .compat import on_win, string_types
 from .. import CondaError
 from .._vendor.auxlib.decorators import memoize
 from .._vendor.toolz import accumulate, concat, take
@@ -43,8 +43,8 @@ def is_path(value):
 
 
 def expand(path):
-    if on_win and PY2:
-        path = ensure_fs_path_encoding(path)
+    # if on_win and PY2:
+    #     path = ensure_fs_path_encoding(path)
     return abspath(expanduser(expandvars(path)))
 
 
@@ -60,7 +60,6 @@ def paths_equal(path1, path2):
     else:
         return abspath(path1) == abspath(path2)
 
-
 @memoize
 def url_to_path(url):
     """Convert a file:// URL to a path.
@@ -72,7 +71,8 @@ def url_to_path(url):
     if not url.startswith("file://"):  # pragma: no cover
         raise CondaError("You can only turn absolute file: urls into paths (not %s)" % url)
     _, netloc, path, _, _ = urlsplit(url)
-    path = unquote(path)
+    from .url import percent_decode
+    path = percent_decode(path)
     if netloc not in ('', 'localhost', '127.0.0.1', '::1'):
         if not netloc.startswith('\\\\'):
             # The only net location potentially accessible is a Windows UNC path
@@ -126,14 +126,21 @@ def explode_directories(child_directories, already_split=False):
 
 
 def pyc_path(py_path, python_major_minor_version):
+    '''
+    This must not return backslashes on Windows as that will break
+    tests and leads to an eventual need to make url_to_path return
+    backslashes too and that may end up changing files on disc or
+    to the result of comparisons with the contents of them.
+    '''
     pyver_string = python_major_minor_version.replace('.', '')
     if pyver_string.startswith('2'):
         return py_path + 'c'
     else:
         directory, py_file = split(py_path)
         basename_root, extension = splitext(py_file)
-        pyc_file = "__pycache__/%s.cpython-%s%sc" % (basename_root, pyver_string, extension)
-        return "%s/%s" % (directory, pyc_file) if directory else pyc_file
+        pyc_file = "__pycache__" + '/' + "%s.cpython-%s%sc" % (
+            basename_root, pyver_string, extension)
+        return "%s%s%s" % (directory, '/', pyc_file) if directory else pyc_file
 
 
 def missing_pyc_files(python_major_minor_version, files):
@@ -281,7 +288,11 @@ def win_path_to_unix(path, root_prefix=""):
     # (C:\msys32\usr\bin\cygpath.exe by MSYS2) to ensure this one is used.
     if not path:
         return ''
-    cygpath = os.environ.get('CYGPATH', 'cygpath.exe')
+    bash = which('bash')
+    if bash:
+        cygpath = os.environ.get('CYGPATH', os.path.join(os.path.dirname(bash), 'cygpath.exe'))
+    else:
+        cygpath = os.environ.get('CYGPATH', 'cygpath.exe')
     try:
         path = subprocess.check_output([cygpath, '-up', path]).decode('ascii').split('\n')[0]
     except Exception as e:
