@@ -54,15 +54,7 @@ conda() {
 
 if [ -z "${CONDA_SHLVL+x}" ]; then
     \export CONDA_SHLVL=0
-    if [ "${_CE_CONDA+x}" == "condax" ]; then
-        if [ "${PATH+x}" == "x" ]; then
-            PATH="$(dirname "$CONDA_EXE")/condabin"
-        else
-            PATH="$(dirname "$(dirname "$CONDA_EXE")")/condabin:${PATH}"
-        fi
-    else
-        PATH="$(dirname "$(dirname "$CONDA_EXE")")/condabin:${PATH:-}"
-    fi
+    PATH="$(\dirname "$(\dirname "$CONDA_EXE")")/condabin${PATH:+":${PATH}"}"
     \export PATH
 
     # We're not allowing PS1 to be unbound. It must at least be set.
